/****************************************************************************************/
/*  BMPPOOL.H                                                                           */
/*                                                                                      */
/*  Author:                                                                             */
/*  Description:                                                                        */
/*                                                                                      */
/*  The contents of this file are subject to the Jet3D Public License                   */
/*  Version 1.02 (the "License"); you may not use this file except in                   */
/*  compliance with the License. You may obtain a copy of the License at                */
/*  http://www.jet3d.com                                                                */
/*                                                                                      */
/*  Software distributed under the License is distributed on an "AS IS"                 */
/*  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See                */
/*  the License for the specific language governing rights and limitations              */
/*  under the License.                                                                  */
/*                                                                                      */
/*  The Original Code is Jet3D, released December 12, 1999.                             */
/*  Copyright (C) 1996-1999 Eclipse Entertainment, L.L.C. All Rights Reserved           */
/*                                                                                      */
/****************************************************************************************/
#ifndef BMP_POOL_DEFINE
#define BMP_POOL_DEFINE

#pragma warning(disable : 4201 4214 4115)
#include <Windows.h>
#include <Windowsx.h>
#pragma warning(default : 4201 4214 4115; disable : 4514)


typedef struct BmpPool BmpPool;

BmpPool* BmpPool_Create();
BOOL BmpPool_Add( BmpPool* pBmpPool,  HINSTANCE	hRes, char* BmpName );

// Added by CJP : 1/8/00
BOOL BmpPool_AddWithName(BmpPool* pBmpPool, HINSTANCE hRes, char* BmpFile, char* BmpName);

HBITMAP BmpPool_Get( BmpPool* pBmpPool, char* BmpName );
BOOL BmpPool_Release( BmpPool* pBmpPool, char* BmpName );
BOOL BmpPool_ReleasePtr( BmpPool* pBmpPool, HBITMAP hBitmap );
void BmpPool_Destroy( BmpPool** hBmpPool );

#endif //BMP_POOL_DEFINE
